#
# Script to manage an ODR or a cluster of ODRs
#

#
# Convert an ODR to an ODR cluster
#
def convertToCluster(odrNodeName,odrServerName,clusterName):
  AdminTask.createCluster(["-clusterConfig", ["-clusterName", clusterName, "-clusterType", "ONDEMAND_ROUTER"], "-convertServer", ["-serverNode", odrNodeName, "-serverName", odrServerName]])
  AdminConfig.save()
  print "Converted ODR '"+odrServerName+"' on node '"+odrNodeName+"' to cluster '"+clusterName+"'"

#
# Create and add a new ODR to an existing ODR cluster
#
def addToCluster(odrNodeName,odrServerName,clusterName):
  AdminTask.createClusterMember(["-clusterName", clusterName, "-memberConfig", ["-memberNode", odrNodeName, "-memberName", odrServerName]])
  AdminConfig.save()
  print "Added ODR '"+odrServerName+"' on node '"+odrNodeName+"' to cluster '"+clusterName+"'"

#
# Insert a rule into a rule set at a particular ruleNumber
#
def insertRule(odrServerOrCluster,ruleSetName,ruleNumber,condition,value):
  ruleSet=getRuleSet(getOdrConfigId(odrServerOrCluster),ruleSetName)
  ruleList=getRuleList(ruleSet)
  ruleNumber=min(ruleNumber,len(ruleList)+1)
  exprList=[]
  valueList=[]
  for rule in ruleList:
    expr=AdminConfig.showAttribute(rule,"expression")
    actionList=AdminConfig.showAttribute(rule,"actions").replace("[","").replace("]","").split(" ")
    for action in actionList:
       val=AdminConfig.showAttribute(action,"value")
    exprList.append(expr)
    valueList.append(val)
    AdminConfig.remove(rule)
  insertIndex = ruleNumber - 1
  for index in range(0,min(insertIndex,len(ruleList))):
    createRule(odrServerOrCluster,ruleSet,ruleSetName,index+1,exprList[index],valueList[index])
  createRule(odrServerOrCluster,ruleSet,ruleSetName,ruleNumber,condition,value)
  for index in range(insertIndex,len(ruleList)):
    createRule(odrServerOrCluster,ruleSet,ruleSetName,index+2,exprList[index],valueList[index])
  AdminConfig.save()
  print "Inserted '"+str(ruleSetName)+" rule #"+str(ruleNumber)

def createRule(odrServerOrCluster,ruleSet,type,ruleNum,condition,value):
   rule=AdminConfig.create("Rule",ruleSet,[["name",ruleNum],["priority",ruleNum],["expression",condition]])
   eles=odrServerOrCluster.split(":")
   if (len(eles) == 1):
     AdminTask.addActionToRule(["-clustername", odrServerOrCluster, "-rulesetName", type, "-ruleName", ruleNum,"-actionName", ruleNum, "-actionType", type, "-actionValue", value])
   else:
     AdminTask.addActionToRule(["-odrname", eles[1], "-nodename", eles[0], "-rulesetName", type, "-ruleName", ruleNum,"-actionName", ruleNum, "-actionType", type, "-actionValue", value])
  
#
# Remove rule number 'ruleNumber' from the rule set
#
def removeRule(odrServerOrCluster,ruleSetName,ruleNumber):
  ruleSet=getRuleSet(getOdrConfigId(odrServerOrCluster),ruleSetName)
  ruleList=getRuleList(ruleSet)
  rule=getRule(ruleSet,ruleNumber)
  if (rule == ""):
    fatal("rule #"+str(ruleNumber)+" was not found")
  AdminConfig.remove(rule)
  exprList=[]
  valueList=[]
  for index in range(ruleNumber,len(ruleList)):
    rule=ruleList[index]
    expr=AdminConfig.showAttribute(rule,"expression")
    actionList=AdminConfig.showAttribute(rule,"actions").replace("[","").replace("]","").split(" ")
    for action in actionList:
       value=AdminConfig.showAttribute(action,"value")
    exprList.append(expr)
    valueList.append(value)
    AdminConfig.remove(rule)
  for index in range(ruleNumber,len(ruleList)):
    createRule(odrServerOrCluster,ruleSet,ruleSetName,index,exprList[index-ruleNumber],valueList[index-ruleNumber])
  AdminConfig.save()
  print "Removed "+str(ruleSetName)+" rule #"+str(ruleNumber)

#
# Get a list of rules.  Remove empty rules.
#
def getRuleList(ruleSet):
  ruleList=AdminConfig.showAttribute(ruleSet,"rules").replace("[","").replace("]","").split(" ")
  for rule in ruleList:
    if (rule == ""):
      ruleList.remove(rule)
  return ruleList

#
# List the rules in a particular rule set
#
def listRules(odrServerOrCluster,ruleSetName):
  ruleSet=getRuleSet(getOdrConfigId(odrServerOrCluster),ruleSetName)
  ruleList=getRuleList(ruleSet)
  for rule in ruleList:
    priority=AdminConfig.showAttribute(rule,"priority")
    condition=AdminConfig.showAttribute(rule,"expression")
    value="None"
    actionList=AdminConfig.showAttribute(rule,"actions").replace("[","").replace("]","").split(" ")
    for action in actionList:
       value=AdminConfig.showAttribute(action,"value")
    print priority+": condition='"+condition+"' value='"+value+"'"

#
# Get a rule from a rule set
#
def getRule(ruleSet,ruleNumber):
  ruleList=AdminConfig.showAttribute(ruleSet,"rules").replace("[","").replace("]","").split(" ")
  for rule in ruleList:
    priority=AdminConfig.showAttribute(rule,"priority")
    if (priority == str(ruleNumber)):
       return rule
  return ""

#
# Get a rule set
#
def getRuleSet(odrConfigId,ruleSetName,createIfNoExist=1):
  ruleSet=AdminConfig.getid(odrConfigId+"Ruleset:"+ruleSetName+"/")
  if (ruleSet == ""):
    if createIfNoExist != 0:
      odrConfig=AdminConfig.getid(odrConfigId)
      ruleSet=AdminConfig.create("Ruleset", odrConfig, [["name", ruleSetName], ["type", "HTTP"], ["defaultContinue", "true"]])
    else:
      usage("rule set '"+ruleSetName+"' does not exist")
  return ruleSet

#
# Get the ODR config ID for an ODR or ODR cluster
#
def getOdrConfigId(odrServerOrCluster):
  eles=odrServerOrCluster.split(":")
  if (len(eles) == 1):
     id="/ServerCluster:"+odrServerOrCluster+"/"
     cfg=AdminConfig.getid(id)
     if (cfg == ""):
        usage("'"+odrServerOrCluster+"' is not an existing cluster")
     return id
  elif (len(eles) == 2):
     id="/Node:"+eles[0]+"/Server:"+eles[1]+"/"
     cfg=AdminConfig.getid(id)
     if (cfg == ""):
        usage("'"+odrServerOrCluster+"' is not an existing ODR")
     return id
  else:
     usage("'"+odrServerOrCluster+"' is neither '<node>:<server>' nor '<cluster>' format");

#
# Set the 'odrNodeName' and 'odrServerName' globals given an ODR path
#
def processOdrPath(odrPath):
  global odrNodeName, odrServerName
  eles=odrPath.split(":")
  if (len(eles) != 2):
    usage("'"+odrPath+"' is not of format <node:odr>")
  odrNodeName = eles[0]
  odrServerName = eles[1]

#
# A fatal error occurred
#
def fatal(msg): 
  print msg   
  sys.exit(1)

#
# Print a usage message
#
def usage(msg): 
  print "ERROR: "+msg+"""
  Supported commands:
    convertToCluster <node:odr> <cluster>
       Convert an existing ODR on node 'node' with name 'odr' to an ODR cluster named <cluster>
    addToCluster <node:odr> <cluster>
       Create a new ODR on node 'node' with name 'odr' and add it to ODR cluster 'cluster'
    insertCustomLogRule <odrServerOrCluster> <ruleNumber> <condition> <logFileFormat>
       Insert a custom log rule into the list at position <ruleNumber>.
    removeCustomLogRule <odrServerOrCluster> <ruleNumber>
       Remove the custom log rule number <ruleNumber>.
    listCustomLogRules <odrServerOrCluster>
       List the custom log rules associated with an ODR or ODR cluster
  where arguments are:
    <node:odr> is the name of the node and ODR (e.g. 'mynode:myodr')
    <cluster> is the name of a cluster
    <odrServerOrCluster> is either <node:odr> or <cluster>
    <ruleNumber> is the number of the rule, from 1 to the total number of rules
    <condition> is the condition (i.e. boolean expression) which must evaluate
        to true in order to trigger the associated custom logging
    <logFileFormat> is specification denoting the file name and format of the
        log entry
  """
  sys.exit(1)

#
# Get argument number 'count'; print a usage message if it doesn't exist.
#
def getarg(count):
  if(len(sys.argv) <= count):
    usage("too few arguments")
  return sys.argv[count].rstrip()

#========================================================================================
#
# Begin main
#
#========================================================================================

global LOG
LOG="log"

command = getarg(0)
if (command == 'convertToCluster'):
  processOdrPath(getarg(1))
  convertToCluster(odrNodeName,odrServerName,getarg(2))
elif (command == 'addToCluster'):
  processOdrPath(getarg(1))
  addToCluster(odrNodeName,odrServerName,getarg(2))
elif (command == 'insertCustomLogRule'):
  insertRule(getarg(1),LOG,int(getarg(2)),getarg(3),getarg(4))
elif (command == 'removeCustomLogRule'):
  removeRule(getarg(1),LOG,int(getarg(2)))
elif (command == 'listCustomLogRules'):
  listRules(getarg(1),LOG)
else:
  usage("'"+command+"' is an unknown command")
